package java.util;

import java.util.function.DoubleConsumer;
import java.util.function.LongConsumer;
import java.util.function.IntConsumer;
import java.util.function.Consumer;

public final class Spliterators
{
    private static final Spliterator<Object> EMPTY_SPLITERATOR;
    private static final Spliterator.OfInt EMPTY_INT_SPLITERATOR;
    private static final Spliterator.OfLong EMPTY_LONG_SPLITERATOR;
    private static final Spliterator.OfDouble EMPTY_DOUBLE_SPLITERATOR;
    
    private Spliterators() {
    }
    
    public static <T> Spliterator<T> emptySpliterator() {
        return Spliterators.EMPTY_SPLITERATOR;
    }
    
    public static Spliterator.OfInt emptyIntSpliterator() {
        return Spliterators.EMPTY_INT_SPLITERATOR;
    }
    
    public static Spliterator.OfLong emptyLongSpliterator() {
        return Spliterators.EMPTY_LONG_SPLITERATOR;
    }
    
    public static Spliterator.OfDouble emptyDoubleSpliterator() {
        return Spliterators.EMPTY_DOUBLE_SPLITERATOR;
    }
    
    public static <T> Spliterator<T> spliterator(Object[] array, int n) {
        return new Spliterators.ArraySpliterator((Object[])Objects.requireNonNull(array), n);
    }
    
    public static <T> Spliterator<T> spliterator(Object[] array, int n, int n2, int n3) {
        checkFromToBounds(((Object[])Objects.requireNonNull(array)).length, n, n2);
        return new Spliterators.ArraySpliterator(array, n, n2, n3);
    }
    
    public static Spliterator.OfInt spliterator(int[] array, int n) {
        return new Spliterators.IntArraySpliterator((int[])Objects.requireNonNull(array), n);
    }
    
    public static Spliterator.OfInt spliterator(int[] array, int n, int n2, int n3) {
        checkFromToBounds(((int[])Objects.requireNonNull(array)).length, n, n2);
        return new Spliterators.IntArraySpliterator(array, n, n2, n3);
    }
    
    public static Spliterator.OfLong spliterator(long[] array, int n) {
        return new Spliterators.LongArraySpliterator((long[])Objects.requireNonNull(array), n);
    }
    
    public static Spliterator.OfLong spliterator(long[] array, int n, int n2, int n3) {
        checkFromToBounds(((long[])Objects.requireNonNull(array)).length, n, n2);
        return new Spliterators.LongArraySpliterator(array, n, n2, n3);
    }
    
    public static Spliterator.OfDouble spliterator(double[] array, int n) {
        return new Spliterators.DoubleArraySpliterator((double[])Objects.requireNonNull(array), n);
    }
    
    public static Spliterator.OfDouble spliterator(double[] array, int n, int n2, int n3) {
        checkFromToBounds(((double[])Objects.requireNonNull(array)).length, n, n2);
        return new Spliterators.DoubleArraySpliterator(array, n, n2, n3);
    }
    
    private static void checkFromToBounds(int n, int n2, int n3) {
        if (n2 > n3) {
            throw new ArrayIndexOutOfBoundsException("origin(" + n2 + ") > fence(" + n3 + ")");
        }
        if (n2 < 0) {
            throw new ArrayIndexOutOfBoundsException(n2);
        }
        if (n3 > n) {
            throw new ArrayIndexOutOfBoundsException(n3);
        }
        return;
    }
    
    public static <T> Spliterator<T> spliterator(Collection<? extends T> collection, int n) {
        return new Spliterators.IteratorSpliterator((Collection<? extends T>)Objects.requireNonNull(collection), n);
    }
    
    public static <T> Spliterator<T> spliterator(Iterator<? extends T> iterator, long n, int n2) {
        return new Spliterators.IteratorSpliterator((Iterator<? extends T>)Objects.requireNonNull(iterator), n, n2);
    }
    
    public static <T> Spliterator<T> spliteratorUnknownSize(Iterator<? extends T> iterator, int n) {
        return new Spliterators.IteratorSpliterator((Iterator<? extends T>)Objects.requireNonNull(iterator), n);
    }
    
    public static Spliterator.OfInt spliterator(PrimitiveIterator.OfInt ofInt, long n, int n2) {
        return new Spliterators.IntIteratorSpliterator((PrimitiveIterator.OfInt)Objects.requireNonNull(ofInt), n, n2);
    }
    
    public static Spliterator.OfInt spliteratorUnknownSize(PrimitiveIterator.OfInt ofInt, int n) {
        return new Spliterators.IntIteratorSpliterator((PrimitiveIterator.OfInt)Objects.requireNonNull(ofInt), n);
    }
    
    public static Spliterator.OfLong spliterator(PrimitiveIterator.OfLong ofLong, long n, int n2) {
        return new Spliterators.LongIteratorSpliterator((PrimitiveIterator.OfLong)Objects.requireNonNull(ofLong), n, n2);
    }
    
    public static Spliterator.OfLong spliteratorUnknownSize(PrimitiveIterator.OfLong ofLong, int n) {
        return new Spliterators.LongIteratorSpliterator((PrimitiveIterator.OfLong)Objects.requireNonNull(ofLong), n);
    }
    
    public static Spliterator.OfDouble spliterator(PrimitiveIterator.OfDouble ofDouble, long n, int n2) {
        return new Spliterators.DoubleIteratorSpliterator((PrimitiveIterator.OfDouble)Objects.requireNonNull(ofDouble), n, n2);
    }
    
    public static Spliterator.OfDouble spliteratorUnknownSize(PrimitiveIterator.OfDouble ofDouble, int n) {
        return new Spliterators.DoubleIteratorSpliterator((PrimitiveIterator.OfDouble)Objects.requireNonNull(ofDouble), n);
    }
    
    public static <T> Iterator<T> iterator(Spliterator<? extends T> spliterator) {
        Objects.requireNonNull(spliterator);
        class Adapter implements Iterator<T>, Consumer<T>
        {
            boolean valueReady;
            T nextElement;
            
            Adapter() {
                this.valueReady = false;
            }
            
            @Override
            public void accept(T nextElement) {
                this.valueReady = true;
                this.nextElement = nextElement;
            }
            
            @Override
            public boolean hasNext() {
                if (!this.valueReady) {
                    spliterator.tryAdvance(this);
                }
                return this.valueReady;
            }
            
            @Override
            public T next() {
                if (!this.valueReady && !this.hasNext()) {
                    throw new NoSuchElementException();
                }
                this.valueReady = false;
                return this.nextElement;
            }
        }
        return new Adapter();
    }
    
    public static PrimitiveIterator.OfInt iterator(Spliterator.OfInt ofInt) {
        Objects.requireNonNull(ofInt);
        class Adapter implements PrimitiveIterator.OfInt, IntConsumer
        {
            boolean valueReady;
            int nextElement;
            
            Adapter() {
                this.valueReady = false;
            }
            
            @Override
            public void accept(int nextElement) {
                this.valueReady = true;
                this.nextElement = nextElement;
            }
            
            @Override
            public boolean hasNext() {
                if (!this.valueReady) {
                    ofInt.tryAdvance(this);
                }
                return this.valueReady;
            }
            
            @Override
            public int nextInt() {
                if (!this.valueReady && !this.hasNext()) {
                    throw new NoSuchElementException();
                }
                this.valueReady = false;
                return this.nextElement;
            }
        }
        return new Adapter();
    }
    
    public static PrimitiveIterator.OfLong iterator(Spliterator.OfLong ofLong) {
        Objects.requireNonNull(ofLong);
        class Adapter implements PrimitiveIterator.OfLong, LongConsumer
        {
            boolean valueReady;
            long nextElement;
            
            Adapter() {
                this.valueReady = false;
            }
            
            @Override
            public void accept(long nextElement) {
                this.valueReady = true;
                this.nextElement = nextElement;
            }
            
            @Override
            public boolean hasNext() {
                if (!this.valueReady) {
                    ofLong.tryAdvance(this);
                }
                return this.valueReady;
            }
            
            @Override
            public long nextLong() {
                if (!this.valueReady && !this.hasNext()) {
                    throw new NoSuchElementException();
                }
                this.valueReady = false;
                return this.nextElement;
            }
        }
        return new Adapter();
    }
    
    public static PrimitiveIterator.OfDouble iterator(Spliterator.OfDouble ofDouble) {
        Objects.requireNonNull(ofDouble);
        class Adapter implements PrimitiveIterator.OfDouble, DoubleConsumer
        {
            boolean valueReady;
            double nextElement;
            
            Adapter() {
                this.valueReady = false;
            }
            
            @Override
            public void accept(double nextElement) {
                this.valueReady = true;
                this.nextElement = nextElement;
            }
            
            @Override
            public boolean hasNext() {
                if (!this.valueReady) {
                    ofDouble.tryAdvance(this);
                }
                return this.valueReady;
            }
            
            @Override
            public double nextDouble() {
                if (!this.valueReady && !this.hasNext()) {
                    throw new NoSuchElementException();
                }
                this.valueReady = false;
                return this.nextElement;
            }
        }
        return new Adapter();
    }
    
    static {
        EMPTY_SPLITERATOR = new EmptySpliterator.OfRef();
        EMPTY_INT_SPLITERATOR = new EmptySpliterator.OfInt();
        EMPTY_LONG_SPLITERATOR = new EmptySpliterator.OfLong();
        EMPTY_DOUBLE_SPLITERATOR = new EmptySpliterator.OfDouble();
    }
    
    private abstract static class EmptySpliterator<T, S extends Spliterator<T>, C>
    {
        EmptySpliterator() {
        }
        
        public S trySplit() {
            return null;
        }
        
        public boolean tryAdvance(C c) {
            Objects.requireNonNull(c);
            return false;
        }
        
        public void forEachRemaining(C c) {
            Objects.requireNonNull(c);
        }
        
        public long estimateSize() {
            return 0L;
        }
        
        public int characteristics() {
            return 16448;
        }
        
        private static final class OfRef<T> extends Spliterators.EmptySpliterator<T, Spliterator<T>, Consumer<? super T>> implements Spliterator<T>
        {
            OfRef() {
            }
        }
        
        private static final class OfInt extends Spliterators.EmptySpliterator<Integer, Spliterator.OfInt, IntConsumer> implements Spliterator.OfInt
        {
            OfInt() {
            }
        }
        
        private static final class OfLong extends Spliterators.EmptySpliterator<Long, Spliterator.OfLong, LongConsumer> implements Spliterator.OfLong
        {
            OfLong() {
            }
        }
        
        private static final class OfDouble extends Spliterators.EmptySpliterator<Double, Spliterator.OfDouble, DoubleConsumer> implements Spliterator.OfDouble
        {
            OfDouble() {
            }
        }
    }
    
    static final class ArraySpliterator<T> implements Spliterator<T>
    {
        private final Object[] array;
        private int index;
        private final int fence;
        private final int characteristics;
        
        public ArraySpliterator(Object[] array, int n) {
            this(array, 0, array.length, n);
        }
        
        public ArraySpliterator(Object[] array, int index, int fence, int n) {
            this.array = array;
            this.index = index;
            this.fence = fence;
            this.characteristics = (n | 0x40 | 0x4000);
        }
        
        @Override
        public Spliterator<T> trySplit() {
            int index = this.index;
            int index2 = index + this.fence >>> 1;
            return (index >= index2) ? null : new Spliterators.ArraySpliterator(this.array, index, this.index = index2, this.characteristics);
        }
        
        @Override
        public void forEachRemaining(Consumer<? super T> consumer) {
            if (consumer == null) {
                throw new NullPointerException();
            }
            Object[] array;
            int fence;
            int index;
            if ((array = this.array).length >= (fence = this.fence) && (index = this.index) >= 0 && index < (this.index = fence)) {
                do {
                    consumer.accept(array[index]);
                } while (++index < fence);
            }
            return;
        }
        
        @Override
        public boolean tryAdvance(Consumer<? super T> consumer) {
            if (consumer == null) {
                throw new NullPointerException();
            }
            if (this.index >= 0 && this.index < this.fence) {
                consumer.accept(this.array[this.index++]);
                return true;
            }
            return false;
        }
        
        @Override
        public long estimateSize() {
            return (long)(this.fence - this.index);
        }
        
        @Override
        public int characteristics() {
            return this.characteristics;
        }
        
        @Override
        public Comparator<? super T> getComparator() {
            if (this.hasCharacteristics(4)) {
                return null;
            }
            throw new IllegalStateException();
        }
    }
    
    static final class IntArraySpliterator implements Spliterator.OfInt
    {
        private final int[] array;
        private int index;
        private final int fence;
        private final int characteristics;
        
        public IntArraySpliterator(int[] array, int n) {
            this(array, 0, array.length, n);
        }
        
        public IntArraySpliterator(int[] array, int index, int fence, int n) {
            this.array = array;
            this.index = index;
            this.fence = fence;
            this.characteristics = (n | 0x40 | 0x4000);
        }
        
        @Override
        public Spliterator.OfInt trySplit() {
            int index = this.index;
            int index2 = index + this.fence >>> 1;
            return (index >= index2) ? null : new Spliterators.IntArraySpliterator(this.array, index, this.index = index2, this.characteristics);
        }
        
        @Override
        public void forEachRemaining(IntConsumer intConsumer) {
            if (intConsumer == null) {
                throw new NullPointerException();
            }
            int[] array;
            int fence;
            int index;
            if ((array = this.array).length >= (fence = this.fence) && (index = this.index) >= 0 && index < (this.index = fence)) {
                do {
                    intConsumer.accept(array[index]);
                } while (++index < fence);
            }
            return;
        }
        
        @Override
        public boolean tryAdvance(IntConsumer intConsumer) {
            if (intConsumer == null) {
                throw new NullPointerException();
            }
            if (this.index >= 0 && this.index < this.fence) {
                intConsumer.accept(this.array[this.index++]);
                return true;
            }
            return false;
        }
        
        @Override
        public long estimateSize() {
            return (long)(this.fence - this.index);
        }
        
        @Override
        public int characteristics() {
            return this.characteristics;
        }
        
        @Override
        public Comparator<? super Integer> getComparator() {
            if (this.hasCharacteristics(4)) {
                return null;
            }
            throw new IllegalStateException();
        }
    }
    
    static final class LongArraySpliterator implements Spliterator.OfLong
    {
        private final long[] array;
        private int index;
        private final int fence;
        private final int characteristics;
        
        public LongArraySpliterator(long[] array, int n) {
            this(array, 0, array.length, n);
        }
        
        public LongArraySpliterator(long[] array, int index, int fence, int n) {
            this.array = array;
            this.index = index;
            this.fence = fence;
            this.characteristics = (n | 0x40 | 0x4000);
        }
        
        @Override
        public Spliterator.OfLong trySplit() {
            int index = this.index;
            int index2 = index + this.fence >>> 1;
            return (index >= index2) ? null : new Spliterators.LongArraySpliterator(this.array, index, this.index = index2, this.characteristics);
        }
        
        @Override
        public void forEachRemaining(LongConsumer longConsumer) {
            if (longConsumer == null) {
                throw new NullPointerException();
            }
            long[] array;
            int fence;
            int index;
            if ((array = this.array).length >= (fence = this.fence) && (index = this.index) >= 0 && index < (this.index = fence)) {
                do {
                    longConsumer.accept(array[index]);
                } while (++index < fence);
            }
            return;
        }
        
        @Override
        public boolean tryAdvance(LongConsumer longConsumer) {
            if (longConsumer == null) {
                throw new NullPointerException();
            }
            if (this.index >= 0 && this.index < this.fence) {
                longConsumer.accept(this.array[this.index++]);
                return true;
            }
            return false;
        }
        
        @Override
        public long estimateSize() {
            return (long)(this.fence - this.index);
        }
        
        @Override
        public int characteristics() {
            return this.characteristics;
        }
        
        @Override
        public Comparator<? super Long> getComparator() {
            if (this.hasCharacteristics(4)) {
                return null;
            }
            throw new IllegalStateException();
        }
    }
    
    static final class DoubleArraySpliterator implements Spliterator.OfDouble
    {
        private final double[] array;
        private int index;
        private final int fence;
        private final int characteristics;
        
        public DoubleArraySpliterator(double[] array, int n) {
            this(array, 0, array.length, n);
        }
        
        public DoubleArraySpliterator(double[] array, int index, int fence, int n) {
            this.array = array;
            this.index = index;
            this.fence = fence;
            this.characteristics = (n | 0x40 | 0x4000);
        }
        
        @Override
        public Spliterator.OfDouble trySplit() {
            int index = this.index;
            int index2 = index + this.fence >>> 1;
            return (index >= index2) ? null : new Spliterators.DoubleArraySpliterator(this.array, index, this.index = index2, this.characteristics);
        }
        
        @Override
        public void forEachRemaining(DoubleConsumer doubleConsumer) {
            if (doubleConsumer == null) {
                throw new NullPointerException();
            }
            double[] array;
            int fence;
            int index;
            if ((array = this.array).length >= (fence = this.fence) && (index = this.index) >= 0 && index < (this.index = fence)) {
                do {
                    doubleConsumer.accept(array[index]);
                } while (++index < fence);
            }
            return;
        }
        
        @Override
        public boolean tryAdvance(DoubleConsumer doubleConsumer) {
            if (doubleConsumer == null) {
                throw new NullPointerException();
            }
            if (this.index >= 0 && this.index < this.fence) {
                doubleConsumer.accept(this.array[this.index++]);
                return true;
            }
            return false;
        }
        
        @Override
        public long estimateSize() {
            return (long)(this.fence - this.index);
        }
        
        @Override
        public int characteristics() {
            return this.characteristics;
        }
        
        @Override
        public Comparator<? super Double> getComparator() {
            if (this.hasCharacteristics(4)) {
                return null;
            }
            throw new IllegalStateException();
        }
    }
    
    public abstract static class AbstractSpliterator<T> implements Spliterator<T>
    {
        static final int BATCH_UNIT = 1024;
        static final int MAX_BATCH = 33554432;
        private final int characteristics;
        private long est;
        private int batch;
        
        protected AbstractSpliterator(long est, int n) {
            this.est = est;
            this.characteristics = (((n & 0x40) != 0x0) ? (n | 0x4000) : n);
        }
        
        @Override
        public Spliterator<T> trySplit() {
            Spliterators.AbstractSpliterator.HoldingConsumer<Object> holdingConsumer = new Spliterators.AbstractSpliterator.HoldingConsumer();
            long est = this.est;
            if (est > 1L && this.tryAdvance(holdingConsumer)) {
                int n = this.batch + 1024;
                if ((long)n > est) {
                    n = (int)est;
                }
                if (n > 33554432) {
                    n = 33554432;
                }
                Object[] array = new Object[n];
                int batch = 0;
                do {
                    array[batch] = holdingConsumer.value;
                } while (++batch < n && this.tryAdvance(holdingConsumer));
                this.batch = batch;
                if (this.est != Long.MAX_VALUE) {
                    this.est -= (long)batch;
                }
                return new Spliterators.ArraySpliterator(array, 0, batch, this.characteristics());
            }
            else {
                return null;
            }
        }
        
        @Override
        public long estimateSize() {
            return this.est;
        }
        
        @Override
        public int characteristics() {
            return this.characteristics;
        }
        
        static final class HoldingConsumer<T> implements Consumer<T>
        {
            Object value;
            
            @Override
            public void accept(T value) {
                this.value = value;
            }
        }
    }
    
    public abstract static class AbstractIntSpliterator implements Spliterator.OfInt
    {
        static final int MAX_BATCH = 33554432;
        static final int BATCH_UNIT = 1024;
        private final int characteristics;
        private long est;
        private int batch;
        
        protected AbstractIntSpliterator(long est, int n) {
            this.est = est;
            this.characteristics = (((n & 0x40) != 0x0) ? (n | 0x4000) : n);
        }
        
        @Override
        public Spliterator.OfInt trySplit() {
            Spliterators.AbstractIntSpliterator.HoldingIntConsumer holdingIntConsumer = new Spliterators.AbstractIntSpliterator.HoldingIntConsumer();
            long est = this.est;
            if (est > 1L && this.tryAdvance(holdingIntConsumer)) {
                int n = this.batch + 1024;
                if ((long)n > est) {
                    n = (int)est;
                }
                if (n > 33554432) {
                    n = 33554432;
                }
                int[] array = new int[n];
                int batch = 0;
                do {
                    array[batch] = holdingIntConsumer.value;
                } while (++batch < n && this.tryAdvance(holdingIntConsumer));
                this.batch = batch;
                if (this.est != Long.MAX_VALUE) {
                    this.est -= (long)batch;
                }
                return new Spliterators.IntArraySpliterator(array, 0, batch, this.characteristics());
            }
            else {
                return null;
            }
        }
        
        @Override
        public long estimateSize() {
            return this.est;
        }
        
        @Override
        public int characteristics() {
            return this.characteristics;
        }
        
        static final class HoldingIntConsumer implements IntConsumer
        {
            int value;
            
            @Override
            public void accept(int value) {
                this.value = value;
            }
        }
    }
    
    public abstract static class AbstractLongSpliterator implements Spliterator.OfLong
    {
        static final int MAX_BATCH = 33554432;
        static final int BATCH_UNIT = 1024;
        private final int characteristics;
        private long est;
        private int batch;
        
        protected AbstractLongSpliterator(long est, int n) {
            this.est = est;
            this.characteristics = (((n & 0x40) != 0x0) ? (n | 0x4000) : n);
        }
        
        @Override
        public Spliterator.OfLong trySplit() {
            Spliterators.AbstractLongSpliterator.HoldingLongConsumer holdingLongConsumer = new Spliterators.AbstractLongSpliterator.HoldingLongConsumer();
            long est = this.est;
            if (est > 1L && this.tryAdvance(holdingLongConsumer)) {
                int n = this.batch + 1024;
                if ((long)n > est) {
                    n = (int)est;
                }
                if (n > 33554432) {
                    n = 33554432;
                }
                long[] array = new long[n];
                int batch = 0;
                do {
                    array[batch] = holdingLongConsumer.value;
                } while (++batch < n && this.tryAdvance(holdingLongConsumer));
                this.batch = batch;
                if (this.est != Long.MAX_VALUE) {
                    this.est -= (long)batch;
                }
                return new Spliterators.LongArraySpliterator(array, 0, batch, this.characteristics());
            }
            else {
                return null;
            }
        }
        
        @Override
        public long estimateSize() {
            return this.est;
        }
        
        @Override
        public int characteristics() {
            return this.characteristics;
        }
        
        static final class HoldingLongConsumer implements LongConsumer
        {
            long value;
            
            @Override
            public void accept(long value) {
                this.value = value;
            }
        }
    }
    
    public abstract static class AbstractDoubleSpliterator implements Spliterator.OfDouble
    {
        static final int MAX_BATCH = 33554432;
        static final int BATCH_UNIT = 1024;
        private final int characteristics;
        private long est;
        private int batch;
        
        protected AbstractDoubleSpliterator(long est, int n) {
            this.est = est;
            this.characteristics = (((n & 0x40) != 0x0) ? (n | 0x4000) : n);
        }
        
        @Override
        public Spliterator.OfDouble trySplit() {
            Spliterators.AbstractDoubleSpliterator.HoldingDoubleConsumer holdingDoubleConsumer = new Spliterators.AbstractDoubleSpliterator.HoldingDoubleConsumer();
            long est = this.est;
            if (est > 1L && this.tryAdvance(holdingDoubleConsumer)) {
                int n = this.batch + 1024;
                if ((long)n > est) {
                    n = (int)est;
                }
                if (n > 33554432) {
                    n = 33554432;
                }
                double[] array = new double[n];
                int batch = 0;
                do {
                    array[batch] = holdingDoubleConsumer.value;
                } while (++batch < n && this.tryAdvance(holdingDoubleConsumer));
                this.batch = batch;
                if (this.est != Long.MAX_VALUE) {
                    this.est -= (long)batch;
                }
                return new Spliterators.DoubleArraySpliterator(array, 0, batch, this.characteristics());
            }
            else {
                return null;
            }
        }
        
        @Override
        public long estimateSize() {
            return this.est;
        }
        
        @Override
        public int characteristics() {
            return this.characteristics;
        }
        
        static final class HoldingDoubleConsumer implements DoubleConsumer
        {
            double value;
            
            @Override
            public void accept(double value) {
                this.value = value;
            }
        }
    }
    
    static class IteratorSpliterator<T> implements Spliterator<T>
    {
        static final int BATCH_UNIT = 1024;
        static final int MAX_BATCH = 33554432;
        private final Collection<? extends T> collection;
        private Iterator<? extends T> it;
        private final int characteristics;
        private long est;
        private int batch;
        
        public IteratorSpliterator(Collection<? extends T> collection, int n) {
            this.collection = collection;
            this.it = null;
            this.characteristics = (((n & 0x1000) == 0x0) ? (n | 0x40 | 0x4000) : n);
        }
        
        public IteratorSpliterator(Iterator<? extends T> it, long est, int n) {
            this.collection = null;
            this.it = it;
            this.est = est;
            this.characteristics = (((n & 0x1000) == 0x0) ? (n | 0x40 | 0x4000) : n);
        }
        
        public IteratorSpliterator(Iterator<? extends T> it, int n) {
            this.collection = null;
            this.it = it;
            this.est = Long.MAX_VALUE;
            this.characteristics = (n & 0xFFFFBFBF);
        }
        
        @Override
        public Spliterator<T> trySplit() {
            Iterator<Object> it;
            long est;
            if ((it = this.it) == null) {
                Iterator<? extends T> iterator = this.collection.iterator();
                this.it = iterator;
                it = iterator;
                long est2 = (long)this.collection.size();
                this.est = est2;
                est = est2;
            }
            else {
                est = this.est;
            }
            if (est > 1L && it.hasNext()) {
                int n = this.batch + 1024;
                if ((long)n > est) {
                    n = (int)est;
                }
                if (n > 33554432) {
                    n = 33554432;
                }
                Object[] array = new Object[n];
                int batch = 0;
                do {
                    array[batch] = it.next();
                } while (++batch < n && it.hasNext());
                this.batch = batch;
                if (this.est != Long.MAX_VALUE) {
                    this.est -= (long)batch;
                }
                return new Spliterators.ArraySpliterator(array, 0, batch, this.characteristics);
            }
            else {
                return null;
            }
        }
        
        @Override
        public void forEachRemaining(Consumer<? super T> consumer) {
            if (consumer == null) {
                throw new NullPointerException();
            }
            Iterator<Object> it;
            if ((it = this.it) == null) {
                Iterator<? extends T> iterator = this.collection.iterator();
                this.it = iterator;
                it = iterator;
                this.est = (long)this.collection.size();
            }
            it.forEachRemaining(consumer);
            return;
        }
        
        @Override
        public boolean tryAdvance(Consumer<? super T> consumer) {
            if (consumer == null) {
                throw new NullPointerException();
            }
            if (this.it == null) {
                this.it = this.collection.iterator();
                this.est = (long)this.collection.size();
            }
            if (this.it.hasNext()) {
                consumer.accept(this.it.next());
                return true;
            }
            return false;
        }
        
        @Override
        public long estimateSize() {
            if (this.it == null) {
                this.it = this.collection.iterator();
                return this.est = (long)this.collection.size();
            }
            return this.est;
        }
        
        @Override
        public int characteristics() {
            return this.characteristics;
        }
        
        @Override
        public Comparator<? super T> getComparator() {
            if (this.hasCharacteristics(4)) {
                return null;
            }
            throw new IllegalStateException();
        }
    }
    
    static final class IntIteratorSpliterator implements Spliterator.OfInt
    {
        static final int BATCH_UNIT = 1024;
        static final int MAX_BATCH = 33554432;
        private PrimitiveIterator.OfInt it;
        private final int characteristics;
        private long est;
        private int batch;
        
        public IntIteratorSpliterator(PrimitiveIterator.OfInt it, long est, int n) {
            this.it = it;
            this.est = est;
            this.characteristics = (((n & 0x1000) == 0x0) ? (n | 0x40 | 0x4000) : n);
        }
        
        public IntIteratorSpliterator(PrimitiveIterator.OfInt it, int n) {
            this.it = it;
            this.est = Long.MAX_VALUE;
            this.characteristics = (n & 0xFFFFBFBF);
        }
        
        @Override
        public Spliterator.OfInt trySplit() {
            PrimitiveIterator.OfInt it = this.it;
            long est = this.est;
            if (est > 1L && it.hasNext()) {
                int n = this.batch + 1024;
                if ((long)n > est) {
                    n = (int)est;
                }
                if (n > 33554432) {
                    n = 33554432;
                }
                int[] array = new int[n];
                int batch = 0;
                do {
                    array[batch] = it.nextInt();
                } while (++batch < n && it.hasNext());
                this.batch = batch;
                if (this.est != Long.MAX_VALUE) {
                    this.est -= (long)batch;
                }
                return new Spliterators.IntArraySpliterator(array, 0, batch, this.characteristics);
            }
            else {
                return null;
            }
        }
        
        @Override
        public void forEachRemaining(IntConsumer intConsumer) {
            if (intConsumer == null) {
                throw new NullPointerException();
            }
            this.it.forEachRemaining(intConsumer);
            return;
        }
        
        @Override
        public boolean tryAdvance(IntConsumer intConsumer) {
            if (intConsumer == null) {
                throw new NullPointerException();
            }
            if (this.it.hasNext()) {
                intConsumer.accept(this.it.nextInt());
                return true;
            }
            return false;
        }
        
        @Override
        public long estimateSize() {
            return this.est;
        }
        
        @Override
        public int characteristics() {
            return this.characteristics;
        }
        
        @Override
        public Comparator<? super Integer> getComparator() {
            if (this.hasCharacteristics(4)) {
                return null;
            }
            throw new IllegalStateException();
        }
    }
    
    static final class LongIteratorSpliterator implements Spliterator.OfLong
    {
        static final int BATCH_UNIT = 1024;
        static final int MAX_BATCH = 33554432;
        private PrimitiveIterator.OfLong it;
        private final int characteristics;
        private long est;
        private int batch;
        
        public LongIteratorSpliterator(PrimitiveIterator.OfLong it, long est, int n) {
            this.it = it;
            this.est = est;
            this.characteristics = (((n & 0x1000) == 0x0) ? (n | 0x40 | 0x4000) : n);
        }
        
        public LongIteratorSpliterator(PrimitiveIterator.OfLong it, int n) {
            this.it = it;
            this.est = Long.MAX_VALUE;
            this.characteristics = (n & 0xFFFFBFBF);
        }
        
        @Override
        public Spliterator.OfLong trySplit() {
            PrimitiveIterator.OfLong it = this.it;
            long est = this.est;
            if (est > 1L && it.hasNext()) {
                int n = this.batch + 1024;
                if ((long)n > est) {
                    n = (int)est;
                }
                if (n > 33554432) {
                    n = 33554432;
                }
                long[] array = new long[n];
                int batch = 0;
                do {
                    array[batch] = it.nextLong();
                } while (++batch < n && it.hasNext());
                this.batch = batch;
                if (this.est != Long.MAX_VALUE) {
                    this.est -= (long)batch;
                }
                return new Spliterators.LongArraySpliterator(array, 0, batch, this.characteristics);
            }
            else {
                return null;
            }
        }
        
        @Override
        public void forEachRemaining(LongConsumer longConsumer) {
            if (longConsumer == null) {
                throw new NullPointerException();
            }
            this.it.forEachRemaining(longConsumer);
            return;
        }
        
        @Override
        public boolean tryAdvance(LongConsumer longConsumer) {
            if (longConsumer == null) {
                throw new NullPointerException();
            }
            if (this.it.hasNext()) {
                longConsumer.accept(this.it.nextLong());
                return true;
            }
            return false;
        }
        
        @Override
        public long estimateSize() {
            return this.est;
        }
        
        @Override
        public int characteristics() {
            return this.characteristics;
        }
        
        @Override
        public Comparator<? super Long> getComparator() {
            if (this.hasCharacteristics(4)) {
                return null;
            }
            throw new IllegalStateException();
        }
    }
    
    static final class DoubleIteratorSpliterator implements Spliterator.OfDouble
    {
        static final int BATCH_UNIT = 1024;
        static final int MAX_BATCH = 33554432;
        private PrimitiveIterator.OfDouble it;
        private final int characteristics;
        private long est;
        private int batch;
        
        public DoubleIteratorSpliterator(PrimitiveIterator.OfDouble it, long est, int n) {
            this.it = it;
            this.est = est;
            this.characteristics = (((n & 0x1000) == 0x0) ? (n | 0x40 | 0x4000) : n);
        }
        
        public DoubleIteratorSpliterator(PrimitiveIterator.OfDouble it, int n) {
            this.it = it;
            this.est = Long.MAX_VALUE;
            this.characteristics = (n & 0xFFFFBFBF);
        }
        
        @Override
        public Spliterator.OfDouble trySplit() {
            PrimitiveIterator.OfDouble it = this.it;
            long est = this.est;
            if (est > 1L && it.hasNext()) {
                int n = this.batch + 1024;
                if ((long)n > est) {
                    n = (int)est;
                }
                if (n > 33554432) {
                    n = 33554432;
                }
                double[] array = new double[n];
                int batch = 0;
                do {
                    array[batch] = it.nextDouble();
                } while (++batch < n && it.hasNext());
                this.batch = batch;
                if (this.est != Long.MAX_VALUE) {
                    this.est -= (long)batch;
                }
                return new Spliterators.DoubleArraySpliterator(array, 0, batch, this.characteristics);
            }
            else {
                return null;
            }
        }
        
        @Override
        public void forEachRemaining(DoubleConsumer doubleConsumer) {
            if (doubleConsumer == null) {
                throw new NullPointerException();
            }
            this.it.forEachRemaining(doubleConsumer);
            return;
        }
        
        @Override
        public boolean tryAdvance(DoubleConsumer doubleConsumer) {
            if (doubleConsumer == null) {
                throw new NullPointerException();
            }
            if (this.it.hasNext()) {
                doubleConsumer.accept(this.it.nextDouble());
                return true;
            }
            return false;
        }
        
        @Override
        public long estimateSize() {
            return this.est;
        }
        
        @Override
        public int characteristics() {
            return this.characteristics;
        }
        
        @Override
        public Comparator<? super Double> getComparator() {
            if (this.hasCharacteristics(4)) {
                return null;
            }
            throw new IllegalStateException();
        }
    }
}
